﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.DecisionList {
  [StorableClass]
  [Item("StringVariable", "")]
  public class StringVariable : Variable<string> {

    [Storable]
    protected IList<string> possibleFeatures;
    public IEnumerable<string> PossibleFeatures {
      get { return possibleFeatures; }
    }

    [StorableConstructor]
    protected StringVariable(bool deserializing) : base(deserializing) { }
    protected StringVariable(StringVariable original, Cloner cloner)
      : base(original, cloner) {
      possibleFeatures = new List<string>(original.possibleFeatures);
    }
    public StringVariable() : base() { }
    public StringVariable(string variableName, IList<string> variableValues)
      : base(variableName) {
      if (variableValues.Count() > 0 && variableValues.Count() != variableValues.Distinct().Count()) {
        throw new ArgumentException("variableValues have to be distinct and there has to be at least one value.");
      }
      possibleFeatures = variableValues;
      attributes = Enumerable.Repeat<bool>(true, possibleFeatures.Count).ToList();
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new StringVariable(this, cloner);
    }

    public override double ComputeTheoryLength() {
      return attributes.Count + attributes.Count(x => x == false);
    }

    public override bool Match(string input) {
      return attributes[possibleFeatures.IndexOf(input)];
    }

    public override void SetToMatch(string variableValue) {
      var valueIndex = possibleFeatures.IndexOf(variableValue);
      if (valueIndex == -1) throw new ArgumentException("variableValue " + variableValue + " is not a possible value of variable " + variableName);

      attributes[valueIndex] = true;
    }
  }
}
